#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "neo.h"

int neo_helper_get_ifindex_from_boardport_generic(neo_device *d, neo_boardport *bp)
{
  return(neo_boardport_get_portnum(bp));
}

int neo_helper_set_boardport_from_ifindex_generic(neo_device *d, neo_boardport *bp, int index)
{
  neo_boardport_create_from_port_int(bp, index);
  return(0);
}

/* Based on 'command', either get the status of a port, enable the
 * port or disable the port.  If the port is an uplink, ask for
 * confirmation before disabling.
 */
int neo_command_port_sed_generic(neo_location *loc, int command)
{
  int ifindex;
  char obj[1024], val[30];
  int a, ret;
  char *devicename;
  neo_device *device;

  device=neo_location_get_first_device(loc);
  devicename=neo_device_get_name(device);
  ifindex=neo_device_get_ifindex_from_location(device, loc);
  if (ifindex<0) {
    printf("Port index out of range for %s\n", devicename);
    return(-1);
  }

  /* Is the Bridge MIB is available? */
  sprintf(obj, ".1.3.6.1.2.1.17.2.15.1.4.%i", ifindex);
  ret=neo_snmp_request_by_variable(devicename, obj, val);
  if (ret) {
    if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
      printf("Could not contact device %s\n", devicename);
      return(-1);
    }

    /* It's not there, fall back to the interfaces section */
    sprintf(obj, ".1.3.6.1.2.1.2.2.1.7.%i", ifindex);
  }
  
  if (command==NEO_COMMAND_PORT_STATUS) {
    ret=neo_snmp_request_by_variable(devicename, obj, val);
    if (ret) {
      if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
        printf("Could not contact device %s\n", devicename);
      } else {
        printf("Error getting port status for %s\n", neo_location_get_instr(loc));
      }
      return(NEO_COMMAND_PORT_IS_UNKNOWN);
    }
    a=atoi(val);
    if (a==1) return(NEO_COMMAND_PORT_IS_ENABLED);
    if (a==2) return(NEO_COMMAND_PORT_IS_DISABLED);
    return(NEO_COMMAND_PORT_IS_UNKNOWN);
  }

  if (command==NEO_COMMAND_PORT_ENABLE) {
    ret=neo_snmp_set_by_variable(devicename, obj, "int", "1");
    if (ret) {
      if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
        printf("Could not contact device %s\n", devicename);
      } else {
        printf("Error getting port status for %s\n", neo_location_get_instr(loc));
      }
      return(NEO_COMMAND_PORT_IS_UNKNOWN);
    }
    return(NEO_COMMAND_PORT_IS_ENABLED);
  }

  if (command == NEO_COMMAND_PORT_DISABLE) {
    neo_locationlist uplink_locations;

    /* get the uplinks */
    neo_locationlist_create(&uplink_locations);
    ret=neo_function_get_uplink(&uplink_locations, devicename);
    if (ret) {
      printf("Warning: failed to get uplink for %s\n", devicename);

      /* do we need to free? */
      return(ret);
    }

    if (neo_locationlist_has_location(&uplink_locations, loc)) {
      printf("Warning!  This port may be the uplink.\n");
      if (!are_you_sure(NULL)) return(NEO_COMMAND_PORT_IS_SILENTERR);
    }
    /* do we need to free? */
    
    ret=neo_snmp_set_by_variable(devicename, obj, "int", "2");
    if (ret) {
      if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
        printf("Could not contact device %s\n", devicename);
      } else {
        printf("Error getting port status for %s\n", neo_location_get_instr(loc));
      }
      return(NEO_COMMAND_PORT_IS_UNKNOWN);
    }
    return(NEO_COMMAND_PORT_IS_DISABLED);
  }
  
  return(0);
}

int neo_command_locate_simple_generic(neo_device *d, neo_searchable *s, neo_locationlist *ans)
{
  neo_location *loc;
  int ret;
  neo_boardport bp;

  /* find the ifIndex the mac address is on */
  ret=neo_function_standardbridge_get_port_from_mac(d, s);
  if (ret==-1) {
    if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
      return(-1);
    } else {
      return(0);
    }
  }

  /* create the location.  If the device has boards, we need to translate */
  loc=malloc(sizeof(neo_location));
  neo_device_set_boardport_from_ifindex(d, &bp, ret);
  neo_location_create_from_boardport_and_device(loc, &bp, neo_device_get_name(d));
  neo_locationlist_append_element(ans, loc);
  neo_boardport_free(&bp);
  return(0);
}

int neo_command_locate_generic(neo_device *d, neo_searchable *s, neo_locationlist *ans, int uplink)
{
  int ret, ret2;
  neo_locationlist first_locations, uplink_locations, result_locations;

  /* first do a normal locate */
  neo_locationlist_create(&first_locations);
  ret=neo_device_command_simplelocate(d, s, &first_locations);
  /* ret=neo_command_locate_simple_generic(d, s, &first_locations); */

  /* if there were no answers, we're done */
  if (neo_locationlist_get_size(&first_locations)<1) return(ret);
  
  /* if we're not filtering the uplink, we're done */
  if (uplink) {
    neo_locationlist_copy(&first_locations, ans);
    /* NEED A SIMPLE FREE OF FIRST_LOCATIONS HERE */
    return(ret);
  }
  
  /* get and filter uplinks */
  neo_locationlist_create(&uplink_locations);
  ret2=neo_function_get_uplink(&uplink_locations, neo_device_get_name(d));
  if (ret2) {
    printf("Warning: failed to get uplink\n");
    neo_locationlist_copy(&first_locations, ans);
    /* NEED A SIMPLE FREE OF FIRST_LOCATIONS HERE */
    return(ret);
  }

  neo_locationlist_create(&result_locations);
  neo_locationlist_prune(&result_locations, &first_locations, &uplink_locations);
  neo_locationlist_copy(&result_locations, ans);

  /* NEED A SIMPLE FREE OF BOTH FIRST_LOCATIONS AND RESULT_LOCATIONS HERE */
  return(ret);
}

int neo_command_device_summary_generic(neo_device *d, neo_location *loc, neo_list *pl)
{
  neo_port *p;
  char buff[1024], obj[1024];
  int i, j, x, y, ret, ifindex;
  neo_locationlist uplink_locations;
  neo_list bp_list;
  neo_location tmploc;
  neo_boardport *thisbp;
  neo_layout *lo;
  char *devicename, *ptr;
  neo_boardport *bp;
  int bridgemib=0;

  devicename=neo_device_get_name(d);

  /* stash the uplinks for later */
  if (neo_global_is_debug(g) || neo_global_is_verbose(g)) printf("Getting uplinks for %s\n", devicename);
  neo_locationlist_create(&uplink_locations);
  ret=neo_function_get_uplink(&uplink_locations, devicename);
  if (ret) {
    printf("Warning: failed to get uplink for %s\n", devicename);
    /* do we need to free? */
    /* return(ret); */
  }

  /* get the layout */
  if (neo_global_is_debug(g) || neo_global_is_verbose(g)) printf("Getting layout for %s\n", devicename);
  neo_device_set_layout(d);
  lo=neo_device_get_layout(d);

  /* set the ifindex ports for the layout */
  neo_function_set_layout_ifindex(d, lo);

  /* if the device seems to use the bridge mib for admin status, then
   * get the bridge index mappings as well */
  strcpy(obj, ".1.3.6.1.2.1.17.2.15.1.4.1");
  ret=neo_snmp_request_by_variable(devicename, obj, buff);
  if (!ret) {
    if (neo_global_is_debug(g)) printf("Using bridgemib for %s\n", devicename);
    neo_function_standardbridge_set_layout_bridgeports(d, lo);
    bridgemib=1;
  }

  /* if a board is specificed in the location, but the device has none,
   * print an error and return.
   */
  if (!neo_layout_has_boards(lo) && neo_location_has_board(loc)) {
    printf("command_device_summary_generic: device %s does not have boards.\n", devicename);
    return(-1);
  }

  /* are we looking at a port list or a board list? */
  if (!neo_layout_has_boards(lo) ||
      (neo_location_has_board(loc) && !neo_location_star_board(loc))) {
    /* port list */

    /* build a list into bp_list containing all ports we're interested in
     */
    neo_list_create(&bp_list);
    if (neo_location_has_port(loc) && !neo_location_star_port(loc)) {
      /* we've got a single port */

      bp=malloc(sizeof(neo_boardport));
      if (!neo_location_has_board(loc)) {
	neo_boardport_create_from_port_int(bp, neo_location_get_portnum(loc));
      } else {
	neo_boardport_create_from_bp_ints(bp, neo_location_get_boardnum(loc), neo_location_get_portnum(loc));
      }
      neo_list_append_element(&bp_list, bp);
    } else {
      /* otherwise many ports */
      if (!neo_layout_has_boards(lo)) {
	/* simple device, all ports */
	j=neo_layout_get_numports(lo);
	for (i=0; i<j; i++) {
	  bp=malloc(sizeof(neo_boardport));
	  neo_boardport_create_from_port_int(bp, i+1);
	  neo_list_append_element(&bp_list, bp);
	}
      } else {
	/* has boards, all ports on that board */
	j=neo_layout_get_board_numports(lo, neo_location_get_boardnum(loc));
	for (i=0; i<j; i++) {
	  bp=malloc(sizeof(neo_boardport));
	  neo_boardport_create_from_bp_ints(bp, neo_location_get_boardnum(loc), i+1);
	  neo_list_append_element(&bp_list, bp);
	}
      }
    }

    /* now traverse that list and fill in info to a portlist */
    y=neo_list_get_size(&bp_list);
    for (x=0; x<y; x++) {
      /* grab the bp, ifindex */
      thisbp=neo_list_get_element(&bp_list, x);
      ifindex=neo_layout_get_port_ifindex(lo, neo_boardport_get_portnum(thisbp));

      /* create a temporary location name for the current bp */
      neo_location_create_from_boardport_and_device(&tmploc, thisbp, devicename);

      /* create a port object, append to list */
      p=malloc(sizeof(neo_port));
      neo_port_create(p);
      neo_portlist_append_element(pl, p);
      
      /* set the port number */
      neo_port_set_number(p, neo_boardport_get_portnum(thisbp));
      
      /* set the port description */
      ptr=neo_device_get_port_descr(d, &tmploc);
      neo_text_replace_str(ptr, "Ethernet", "Eth");
      neo_text_replace_str(ptr, "ethernet", "Eth");
      neo_text_replace_str(ptr, "Fast ", "Fst");
      neo_text_replace_str(ptr, "fast ", "Fst");
      neo_text_replace_str(ptr, "multi mode", "mm");
      neo_text_replace_str(ptr, "single mode", "sm");
      neo_text_replace_str(ptr, "gigabit", "Gig");
      neo_text_replace_str(ptr, "Gigabit", "Gig");

      neo_port_set_descr(p, ptr);
      free(ptr);

      /* set uplinks */
      if (neo_locationlist_has_location(&uplink_locations, &tmploc)) {
	neo_port_set_uplink(p, 1);
      }
      
      /* set link status */
      /* interfaces.ifTable.ifEntry.ifOperStatus.%i */
      sprintf(obj, ".1.3.6.1.2.1.2.2.1.8.%i", ifindex);
      ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, buff);
      if (ret) {
	continue;
      } else {
	if (atoi(buff)==1) {
	  neo_port_set_link_up(p);
	} else if (atoi(buff)==2) {
	  neo_port_set_link_down(p);
	} else {
	  neo_port_set_link_unknown(p);
	}
      }
      
      /* speed */
      /* interfaces.ifTable.ifEntry.ifSpeed.%i */
      sprintf(obj, ".1.3.6.1.2.1.2.2.1.5.%i", ifindex);
      ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, buff);
      if (ret) {
	continue;
      } else {
	neo_port_set_speed(p, atoi(buff));
      }

      /* admin status */
      if (bridgemib) {
	sprintf(obj, ".1.3.6.1.2.1.17.2.15.1.4.%i", neo_layout_get_port_bridgeport(lo, neo_boardport_get_portnum(thisbp)));
      } else {
	sprintf(obj, ".1.3.6.1.2.1.2.2.1.7.%i", ifindex);
      }
      ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, buff);
      if (ret) {
	continue;
      } else {
	if (atoi(buff)==1) {
	  neo_port_set_admin(p, 1);
	} else if (atoi(buff)==2) {
	  neo_port_set_admin(p, 0);
	} else {
	  neo_port_set_admin(p, -1);
	}
      }
      neo_location_free(&tmploc);
    }

    /* neo_list_free_all(&bp_list, &neo_boardport_free); */
    return(NEO_TYPE_PORTLIST);
  } else {

    return(NEO_TYPE_BOARDLIST);
  }
}

int neo_command_stats_generic(neo_location *loc, neo_portlist *pl)
{
  neo_statstransfer *st, *a, *b;
  neo_device *d;
  neo_list *stlist1, *stlist2, *stlistcur;
  neo_port *p;
  neo_stats *s;
  neo_sleeper sleeper;
  int i, j, portnum, ret, done;

  d=neo_location_get_first_device(loc);
  ret=neo_command_device_summary_generic(d, loc, pl);
  j=neo_portlist_get_size(pl);
  stlist1=malloc(sizeof(neo_list));
  stlist2=malloc(sizeof(neo_list));
  neo_list_create(stlist1);
  neo_list_create(stlist2);

  neo_sleeper_create(&sleeper, neo_global_get_statsdelay(g));
  /* do it twice first with stlistcur=stlist1 then stlistcur=stlist2 */
  done=0;
  for (stlistcur=stlist1,done=0; done<2; stlistcur=stlist2, done++) {
    if (done==0) {
      printf("Getting first set of stats...\n");
    } else {
      printf("Getting second set of stats...\n");
    }
    for (i=0; i<j; i++) {
      p=neo_portlist_get_element(pl, i);
      portnum=neo_port_get_number(p);

      st=malloc(sizeof(neo_statstransfer));
      neo_list_append_element(stlistcur, st);

      neo_statstransfer_init(st);

      neo_function_standardbridge_stats(d, st, portnum);
    }

    /* do the necessary delay here */
    if (done==0) {
      neo_sleeper_dowait(&sleeper);
    }
  }

  s=malloc(sizeof(neo_stats));
  /* now for each port do the stats transfer calculation */
  for (i=0; i<j; i++) {
    p=neo_portlist_get_element(pl, i);
    a=neo_list_get_element(stlist1, i);
    b=neo_list_get_element(stlist2, i);

    /* calculate and set for port */
    neo_statstransfer_compute_stats(a, b, s);
    neo_port_set_stats(p, s);
  }

  /* free the statstransfer lists */
  return(NEO_TYPE_PORTLIST);
}

int neo_command_port_search_generic(neo_location *loc)
{
  neo_device *d;
  int index, ret;

  if (!neo_location_has_port(loc)) {
    printf("You must specify a port to search.\n");
    return(-1);
  }

  d=neo_location_get_first_device(loc);
  index=neo_device_get_ifindex_from_location(d, loc);
  ret=neo_function_standardbridge_portsearch(d, index);
  return(0);
}

void neo_command_device_info_generic(neo_device *d, int type)
{
  
  if (type==NEO_COMMAND_DEVICE_INFO_UPS || type==NEO_COMMAND_DEVICE_INFO_POWER) {
    neo_command_device_info_ups_generic(d);
  }
}

void neo_command_device_info_ups_generic(neo_device *d)
{
  int ret, inputlines, outputlines, i, j;
  float f;
  char val[1024], obj[1024], timebuff[1024];

  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.1.1.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    printf("  Type             : %s\n", val);
  }

  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.1.5.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    printf("  Name             : %s\n", val);
  }

  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.6.1.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    printf("  # Alarms Present : %i\n", atoi(val));
  }
  
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.4.1.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    i=atoi(val);
    printf("  Status           : ");
    if (i==1) {
      printf("Other\n");
    } else if (i==2) {
      printf("None\n");
    } else if (i==3) {
      printf("Normal\n");
    } else if (i==4) {
      printf("Bypass\n");
    } else if (i==5) {
      printf("Battery\n");
    } else if (i==6) {
      printf("Booster\n");
    } else if (i==7) {
      printf("Reducer\n");
    } else {
      printf("Unknown value (%i)\n",i);
    }
  }
  
  printf("\n");
  
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.2.4.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    i=atoi(val);
    printf("  Battery Capacity : %i%%\n", i);
  }
  
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.2.3.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    neo_timeformat(timebuff, atoi(val)*60);
    printf("  Battery Run Time : %s\n", timebuff);
  }
  
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.2.2.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    val[strlen(val)-2]='\0'; /* chop off last two digits */
    neo_timeformat(timebuff, atoi(val));
    printf("  Time On Battery  : %s\n", timebuff);
  }
  
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.2.5.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    f=atol(val)/10;
    printf("  Battery Voltage  : %-.1f V\n", f);
  }
  
  
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.2.6.0", val);
  if (ret) {
    printf("Error getting info.\n");
  } else {
    f=atol(val)/10;
    printf("  Battery Current  : %-.1f V\n", f);
  }

  printf("\n");

  inputlines=0;
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.3.2.0", val);
  if (ret) {
    printf("Error getting number of input lines.\n");
  } else {
    inputlines=atoi(val);
  }
  
  for (j=0; j<inputlines; j++) {
    sprintf(obj, ".1.3.6.1.2.1.33.1.3.3.1.3.%i", j+1);
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Input %i Voltage  : %i.0 V\n", j+1, atoi(val));
    }
    
    sprintf(obj, ".1.3.6.1.2.1.33.1.3.3.1.4.%i", j+1);
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Input %i Current  : %-.1f A\n", j+1, atof(val)/10);
    }
    
    
    sprintf(obj, ".1.3.6.1.2.1.33.1.3.3.1.2.%i", j+1);
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Input %i Freq.    : %-.1f Hz\n", j+1, atof(val)/10);
    }
    
  }
  
  printf("\n");
  
  outputlines=0;
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.4.3.0", val);
  if (ret) {
    printf("Error getting number of output lines.\n");
  } else {
    outputlines=atoi(val);
  }
  
  for (j=0; j<outputlines; j++) {
    sprintf(obj, ".1.3.6.1.2.1.33.1.4.4.1.2.%i", j+1);
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Output %i Voltage : %i V\n", j+1, atoi(val));
    }
    
    sprintf(obj, ".1.3.6.1.2.1.33.1.4.4.1.3.%i", j+1);
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Output %i Current : %-.1f A\n", j+1, atof(val)/10);
    }

    /* yes, this is the same every time */
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.33.1.4.2.0", val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Output %i Freq.   : %-.1f Hz\n", j+1, atof(val)/10);
    }
    
    sprintf(obj, ".1.3.6.1.2.1.33.1.4.4.1.5.%i", j+1);
    ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
    if (ret) {
      printf("Error getting info.\n");
    } else {
      printf("  Output %i Load    : %i%%\n", j+1, atoi(val));
    }
    
  }
}

char *neo_helper_device_get_port_descr_generic(neo_device *d, neo_location *loc)
{
  int ifindex, ret;
  char obj[1024], val[1024];

  ifindex=neo_device_get_ifindex_from_location(d, loc);
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.2.%i", ifindex);
	 
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
  if (ret) {
    return(strdup("Generic Port"));
  }
  return(strdup(val));
}

char *neo_helper_device_get_board_descr_generic(neo_device *d, neo_location *loc)
{
  return(strdup("Generic Board"));
}

int neo_helper_device_set_layout_generic(neo_device *d)
{
  neo_layout *lo;
  int ret;
  char val[1024], *devicename;

  devicename=neo_device_get_name(d);
  lo=neo_device_get_layout(d);
  
  ret=neo_snmp_request_by_variable(devicename, ".1.3.6.1.2.1.2.1.0", val);
  if (ret) {
    printf("helper_set_layout_generic: Could not get number of interfaces from %s\n", devicename);
    /* neo_error_print(neo_error); */
    return(-1);
  }
  neo_layout_set_num_ports(lo, atoi(val));
  neo_device_set_layout_set(d);
  return(0);
}
